#!perl

# AutoClient.pl - the AutoNap remote control
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap $Revision: 1.2 $
# last change made $Date: 2001/04/20 08:21:32 $

use strict;
use IO::Socket;
use Term::ReadLine;
use Term::ReadKey;
use Term::ANSIColor;
use Data::Dumper;
my ($remote, $port, $iaddr, $paddr, $proto, $line) = (shift, shift);
if ($port and ($port =~ /\D/ or $port < 1025)) {
	die "Error: port must be numeric and in the range of 1025 and 65535";
}

$remote ||= 'localhost';
$port   ||= 9876;  

if ($remote eq '-h') {
	print <<HELP;
	
AutoClient.pl - AutoNap remote control
usage: AutoClient.pl [ip] [port]
       ip and port default to localhost and 9876

HELP
	exit(0);
}

print "connecting to $remote on $port...";

my $server = IO::Socket::INET->new(Proto     => "tcp",
	     			   PeerAddr  => $remote,
				   PeerPort  => $port) or die "failed: $!";
$server->autoflush(1);

print "success.\n";

# create terminal
our $term = new Term::ReadLine 'AutoClient';
our $prompt = "AutoClient-> ";
our $OUT = $term->OUT || \*STDOUT;


sub output {
	# copy the socket to standard output
	my @reply;
	my $printed = 0;
	while (defined (my $line = <$server>)) {
		$printed = 1;
		if ($line eq "EOL\n") {
			$server->flush;
			last;
		}
		elsif ($line eq "START_PAGING\n") {
			ReadMode 4;
			my $key;
			while (not defined($key = ReadKey(-1))) { 
				# wait
			}
			syswrite $server, $key, 1;
			ReadMode 0;
		}
		elsif ($line eq "LINE_PLEASE\n") {
			my $num = $term->readline("Give song number: ");
			syswrite $server, $num."\n", length($num."\n");
		}
		else {
			print $OUT $line;
		}
	}
}

#copy standard input to the socket
while ( defined ($_ = $term->readline($prompt)) ) {
	$term->addhistory($_) if /\S/;
	print $server $_."\n" if /\S/;
	quit_client() if $_ =~ /^exit$/;
	quit_all()    if $_ =~ /^quit$/;
	output()      if $_ !~ /^\s*$/;
}

sub quit_client {
	exit(0);
	
}

sub quit_all {
	print $server "quit";
	exit(0);
}
