# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap robot module $Revision: 1.15 $
# last change made $Date: 2001/05/15 06:07:39 $


package AutoLib::Auto;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw( &auto_init );
	%EXPORT_TAGS  = ( );

	}

use strict;
use warnings;
use AutoLib::Shell;
use vars @EXPORT_OK;

use MP3::Napster;

use constant FIRST    => 1;
use constant SRCH     => 2;
use constant START_D  => 3;
use constant END_D    => 4;
use constant ERR      => 5;
use constant VERBATIM => 6;

#-----------------
# Global variables
#-----------------

our 
	( $NAP, $DIR, $FILE, 
	  $RND, @RND_PATTERN, @RND_EX,
	  $PING,
      $MAX_DOWNLOADS, $MAX_SIMULTAN, $TIMEOUT,
	  @PATTERN,
      $LINESPEED, $MIN_LENGTH, $MAX_LENGTH, $MIN_BITRATE, $MAX_BITRATE,
	  @FOUND, @PINGED
	);


#----------------------
# some status variables
#----------------------

# in this we store whether a download has been successful
# if yes, we can skip to the next pattern-section
# if not, to the next song of the same section
my $DOWNLOAD_SUCCESS = 0; 

# needed in order to prevent starting a new download
# before the actual callback has been invoked
my $READY_FOR_NEXT = 1;

my $TOTAL_DOWNLOADS = 0;

#-----------------------
# time stuff for logfile
#-----------------------
our ($S_SEC, $S_MIN, $S_HOUR, $S_DAY, $S_MON, $S_YEAR) = localtime(time);
our $START = time;


#------------------------------------
# Keywords and variables for 'script'
#------------------------------------

our %VARS =
	(
		pattern   => 1,
		bitrate   => 1,
		linespeed => 1,
		length    => 1,
	);

our %SPEEDS =
	(
		'UNKOWN' => 0,
		'14K'    => 1,
		'28K'    => 2,
		'33K'    => 3,
		'56K'    => 4,
		'64K'    => 5,
		'128K'   => 6,
		'DSL'    => 7,
		'CABLE'  => 8,
		'T1'     => 9,
		'T3'     => 10
	);
		
#------------------------------
# preparations and control flow 
# are made here
#------------------------------

sub auto_init {

	($NAP, $DIR, $FILE, @RND_PATTERN) = @_;
	print $DIR."\n\n";
	log_it(FIRST);

	parse();
	$NAP->transfer_timeout($TIMEOUT);
	$NAP->callback(TRANSFER_STARTED, \&CLB_transfer_start);
	$NAP->callback(TRANSFER_DONE, \&CLB_transfer_end);
	$NAP->callback(PONG, \&CLB_pong);	
	# perform sequence: search -> ping/rate -> get
	search();
	ping();		
	log_it(SRCH);
	get();

}




#-------------------------------
# parse relevant information out
# of configfile
#-------------------------------

sub parse {

	open CONF, "<$FILE" 
		or die "Error: $FILE couldn't be opened for reading: $!";

	# at the beginning we are neither in [random] nor in [auto] section;
	my ($random, $auto) = (0, 0);
	
	while (<CONF>) {
		my $tmp;
		undef $tmp;
		chop; 
		
		($random, $auto) = (1, 0) if /^\[random\]/;
		($random, $auto) = (0, 1) if /^\[auto\]/;

		if ($random) {
		
			# rnd
			$RND = 1 if /rnd=yes/;

			# rnd_exclude_pattern
			(undef, @RND_EX) = split /=/ if /^rnd_exclude_patterns/;
		
		}

		elsif ($auto) {
			
			if (/^auto_ping/) {
				(undef, $PING) = split /=/;
				if ($PING eq 'yes') { $PING = 1 } 
				else                { $PING = 0 }
			}

			(undef, $MAX_DOWNLOADS) = split /=/ if /^auto_max_downloads/;
			
			(undef, $MAX_SIMULTAN)  = split /=/ if /^auto_max_simultan/;
		
			(undef, $TIMEOUT)       = split /=/ if /^auto_timeout/;

			(undef, $tmp)           = split /=/ if /^auto_pattern/;
			@PATTERN = split /::/, $tmp if $tmp;
			
			if (/^auto_bitrate/) {
				(undef, $tmp) = split /=/;
				if ($tmp and $tmp =~ /-/) { 
					($MIN_BITRATE, $MAX_BITRATE) = split /-/, $tmp;
				}
				else { 
					$MIN_BITRATE = $tmp;
				}
			}
			
			(undef, $LINESPEED) 	= split /=/ if /^auto_linespeed/;

			(undef, $tmp) 		= split /=/ if /^auto_length/;
			if ($tmp and $tmp =~ /-/) { 
				($MIN_LENGTH, $MAX_LENGTH) = split /-/, $tmp; 
			}
			else { 
				$MIN_LENGTH = $tmp; 
			}		
			
		}

	}
	close CONF;

	$PING          ||= 0;
	$MAX_DOWNLOADS ||= 0;
	$MAX_SIMULTAN  ||= 1;
	$TIMEOUT       ||= 20;
	$MIN_BITRATE   ||= 128;
	$MAX_BITRATE   ||= 160;
	$LINESPEED     ||= 9;
	$MIN_LENGTH    ||= "2:00"; $MIN_LENGTH = len_to_secs($MIN_LENGTH);
	$MAX_LENGTH    ||= "5:00"; $MAX_LENGTH = len_to_secs($MAX_LENGTH);
				
}

#---------------------
# make searches
# and trying all
# permutations for
# pattern and bitrates
#---------------------

sub search {
	
	my $pattern;
	my @bitrates;
	my @result_record;
	
	# have a look at what parameters are defined
	# if not set some sensible defaults

	print "$MIN_BITRATE $MAX_BITRATE $LINESPEED $MIN_LENGTH $MAX_LENGTH\n";

	push @bitrates, $MIN_BITRATE;
	push @bitrates, $MAX_BITRATE if $MIN_BITRATE != $MAX_BITRATE;

	print "$MIN_BITRATE $MAX_BITRATE $LINESPEED $MIN_LENGTH $MAX_LENGTH\n";
	
	# if user prefers random searches
	if ($RND) {
		undef @PATTERN;
		my $len = scalar(@RND_PATTERN);
		for (1 .. 3) {
			# get a random number from /dev/urandom
			my $which = sprintf "%d", rand $len;
			
			push @PATTERN, $RND_PATTERN[$which];
		}
	}
	
	for my $artist (@PATTERN) {
		my @tmp;
		print "Artist: $artist\n";
		for my $bits (@bitrates) {
			my @s = $NAP->search (artist  => $artist,
					       	      bitrate => "at least $bits");
			push @tmp, @s ;
		}
		push @result_record, [ @tmp ];
	}
	filter(@result_record);
}


#-----------------------
# Examine whether search
# hits the user-defined 
# criteria
#-----------------------
sub filter {

	my @cur_result = @_;
	
	# $p is an array-reference
	for my $p (@cur_result) {
	
		my @tmp;

		for my $hit (@$p) {
		
			if ( $hit->length >= $MIN_LENGTH && $hit->length <= $MAX_LENGTH and
			$hit->bitrate >= $MIN_BITRATE && $hit->bitrate <= $MAX_BITRATE  and
			$hit->link >= $LINESPEED ) {
				#check rnd_exclude_patterns
				if ($RND) {
					for my $r (@RND_EX) {
						if ($hit->name !~ /$r/) {
							push @tmp, $hit;
							last;
						}
					}
				}
				else {
					push @tmp, $hit;
				}
			}
		}
		push @FOUND, [ @tmp ];
	}
}
	
#-----------------------
# ping all owners
# and wrap songs into
# a new hash containing
# ping responses as well
#-----------------------
sub ping {

	my @to_be_sorted;

	for my $p (@FOUND) {
		
		my @pinged_tmp = undef;
		
		for my $s (@$p) {
			
			my %psong;
			$psong{song} = $s;
		
			my $start = time;
			my $end   = $start;
			
			if ($PING) {
				$psong{song}->owner->ping;
				$end = time;
			}
		
			$psong{reply} = $end - $start;
			push @pinged_tmp, { %psong };

		}

		push @to_be_sorted, [ @pinged_tmp ];
	}

	for my $p (@to_be_sorted) {
		
		# sort by ping-reply
		my @sorted_tmp = sort
			{ $$AutoLib::Auto::a{reply} <=> $$AutoLib::Auto::b{reply} } @$p;
		push @PINGED, [ @sorted_tmp ];
	}
	
	for my $arr (@PINGED) {
		for my $song (@$arr) {
			print $$song{song}->name."\n" if defined $$song{song};
		}
	}
}

#---------------------------------
# provides download functionality:
# This is yet the most primitive
# solution so it will need 
# extensions later
#---------------------------------

sub get {

	# we might need this later when 
	# implementing more refined getting
	my @args = @_;
	
	log_it(VERBATIM, "Starting downloads:\n".
	                 "------------------>>\n");	

	for my $i (@PINGED) {
	
		$DOWNLOAD_SUCCESS = undef;
		
		for my $song (@$i) {

			last if not $TOTAL_DOWNLOADS < $MAX_DOWNLOADS 
					and $MAX_DOWNLOADS > 0;
			next if not defined %$song;
			
			my $handle = $DIR."/".$$song{song}->name;
			
			log_it(VERBATIM, "Trying: ".$$song{song});
			log_it(VERBATIM, "Initiating download");
			
			while (not $READY_FOR_NEXT) {
				# wait;
			}
			
			$NAP->download($$song{song}, $handle);
			
			print "Currently running downloads: ".scalar($NAP->downloads)."\n";
			print "$MAX_SIMULTAN at a time allowed.\n";
			unless (scalar($NAP->downloads) < $MAX_SIMULTAN) {
				$READY_FOR_NEXT = 0;
				block_next();
			}

			last if $DOWNLOAD_SUCCESS and not $RND;
			
		}
		
		log_it(VERBATIM, "Entering next section if there is any.");			

	}
}

#--------------------
# All logging is done
# in here
#--------------------

sub log_it {

	open LOG, ">>auto.log";
	$| = 1;

	my ($event, $msg) = @_;
	my $str = "**".cur_time().": ";
	print LOG $str;

	if ($event == FIRST) {
		print LOG "Session started\n";
	}

	if ($event == SRCH) {
		print LOG "Searched and pinged in ".(time() - $START)." secs\n";
		print LOG "I found:\n".
		          "------>>\n";
		for my $i (@PINGED) {
			for my $s (@$i) {
				printf LOG "%-50.40s%-2.2s secs\n",
				           $$s{song}, $$s{reply} unless not defined %$s;
			}
			print LOG "-" x 57;
			print LOG "\n";
		}
	}

	if ($event == VERBATIM) {
		print LOG "$msg\n";
	}

	close LOG;

}
		

#-------------------------
# implements a while-loop
# which will block until
# a download has completed
#-------------------------

sub block_next {
	my $start_t = time;
	print "waiting for ".scalar($NAP->downloads)." downloads to finish...\n";

	while (scalar($NAP->downloads) >= $MAX_SIMULTAN) {
		my ($event,$download) = $NAP->wait_for(TRANSFER_DONE);
	}					 
}

#-------------------------------
# This will format the current
# time properly
#-------------------------------

sub cur_time {
	my @t = localtime(time());	
	return sprintf "%d/%d/%d at %02d:%02d:%02d", 
	                $t[5]+1900, $t[4]+1, $t[3],
			$t[2], $t[1], $t[0];
}

#-------------------------------
# we need our own callbacks here
#-------------------------------

sub CLB_transfer_start {
	my ($nap, $code, $trans) = @_;
	log_it(VERBATIM,  "Starting ".$trans->asString."\nNow waiting...");
}


sub CLB_transfer_end {
	my ($nap, $code, $trans) = @_;
	log_it(VERBATIM, $trans->asString.":\n".
	                 $trans->status);
	$READY_FOR_NEXT = 1;
	if ($trans->status eq 'transfer complete') {
		$DOWNLOAD_SUCCESS = 1;
		$TOTAL_DOWNLOADS++;
	}
}
sub CLB_pong {
	my ($nap, $code, $msg) = @_;
}
1;
