# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap command processing module $Revision: 1.7 $
# last change made $Date: 2001/05/22 17:40:39 $

package AutoLib::CommandProcessor;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(&get_user_input &auto_nap_shell $term);
	%EXPORT_TAGS  = ();

	}
	
use strict;
use vars     @EXPORT_OK;

use Term::ReadLine;
use Term::ANSIColor qw (:constants);
use AutoLib::Shell;

our @FOUND;
our $term;

sub get_user_input {
	
	$term = new Term::ReadLine 'AutoClient';
	our $prompt = "AutoClient-> ";
	our $OUT = $term->OUT || \*STDOUT;

	while ( defined ($_ = $term->readline($prompt)) ) {
	        $term->addhistory($_) if /\S/;
		auto_nap_shell("junk", $_);
	}

}
						
	

#-----------------
# the actual shell
#-----------------
sub auto_nap_shell {
        shift;
		my $input = shift;
		my $out   = shift || \*STDOUT;
		$input =~ s/\s+$//;
		
	   	# cmd_help
		if ($input =~ /^help/){
			$input  =~ s/(help)(.*)/$2/;
			my @output = cmd_help($input);
			printf $out BLUE."-----\n";
			printf $out @output;
			printf $out BLUE."\n-----".RESET."\n";
		}

		# cmd_quit
		elsif ($input =~ /^quit$|^exit$/) {
	    		cmd_quit();
		}
									
		# cmd_get
		elsif ($input =~ /^get$|^get /) {
		 	$input =~ s/(get)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_get($input, @FOUND);
			}
		}

		# cmd_search
		elsif ($input =~ /^search$|^search /) {
			$input =~ s/(search)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				my @tmp = cmd_search($input);
				@FOUND = @tmp unless not defined $tmp[0];
			}
		}
		
		# cmd_rsearch
		elsif ($input =~ /^rsearch$|^rsearch /) {
			$input =~ s/(rsearch)(.*)/$2/;
				my @tmp = cmd_rsearch($input);
				@FOUND = @tmp unless not defined $tmp[0];
		}
		
		# cmd_browse
		elsif ($input =~ /^browse$|^browse /) {
		       	$input =~ s/(browse)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
		else {
			my @tmp = cmd_browse($input);
			@FOUND = @tmp unless not defined $tmp[0];
			}
		}

                # cmd_ping
		elsif ($input =~ /^ping$|^ping /) {
			$input =~ s/(ping)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_ping($input);
			}
		}

		# cmd_ddown
		elsif ($input =~ /^ddown$|^ddown /) {
			$input =~ s/(ddown)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_ddown($input);
			}
		}

		# cmd_downs
		elsif ($input =~ /^downs/) {
	    		cmd_downs();
		}

		# cmd_dup
		elsif ($input =~ /^^dup$|dup /) {
			$input =~ s/(dup)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_dup($input);
			}
		}

		# cmd_ups
		elsif ($input =~ /^ups/) {
			cmd_ups();
		}

		# cmd_check
		elsif ($input =~ /^check$|^check /) {
		     	$input =~ s/(check )(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_check($input, @FOUND);
			}
		}

		# cmd_last
		elsif ($input =~ /^last/) {
	   		$input =~ s/(last )(.*)/$2/;
			my @tmp = cmd_last($input);
			@FOUND = @tmp unless not defined $tmp[0];
		}

		# cmd_blast
		elsif ($input =~ /^blast/) {
			$input =~ s/(blast )(.*)/$2/;
			my @tmp = cmd_blast($input);
			@FOUND = @tmp unless not defined $tmp[0];
		}

		# cmd_stream
		elsif ($input =~ /^stream$|^stream /) {
			$input =~ s/(stream)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_stream($input, @FOUND);
			}
		}

		#cmd_whois
		elsif ($input =~ /^whois$|^whois /) {
			$input =~ s/(whois)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_whois($input);
			}
		}

		#cmd_raw
		elsif ($input =~ /^raw$|^raw /) {
			$input =~ s/(raw)(.*)/$2/;
			if ($input =~ /^\s+$/ or not $input) {
				missing_arg($out);
			}
			else {
				cmd_raw($input);
			}
		}

		# shell commands
		elsif ($input =~ /^!/) {
			$input =~ s/(!)(.*)/$2/;
			eval { system($2); };
		}

		# nothing
		elsif ($input =~ /^\s*$/) {
	   		# do nothing
		}

		# cmd_unkown
		else {
			printf $out BLUE."-----\n".
			            RED."$input - Unkown command. ".
			                "Type 'help' in case you feel lost.".
				        BLUE."\n-----".RESET."\n";
		}
}

#---------------------
# standard error reply
#---------------------
sub missing_arg {
	my $out = shift;
        printf $out BLUE."-----\n".
	                RED."Argument(s) required".
	                BLUE."\n-----".RESET."\n";			     
}

1;									
