# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap login/register module $Revision: 1.11 $
# last change made $Date: 2001/05/25 05:44:00 $

package AutoLib::Login;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(&log_in &login_to_server 
					   &register_account &get_user &get_pass);
	%EXPORT_TAGS  = ();

	}
	
use strict;
use vars     @EXPORT_OK;
use Term::ANSIColor qw (:constants);
use MP3::Napster;
use AutoLib::Shell qw(%LSPEED);


#-----------------
# Login to network
#-----------------
sub log_in {
	
	my ($network, $local_port, $meta, $user, $pass, $down_dir, @peers) = @_;
	my $nap;

	for my $p (@peers) {
		my (
			$p_ip,
			$p_port,
			$p_network,			            
			$p_users,
			$p_songs,
			$p_gigs,
			$p_host,
		   )=	
		   (
			$p->{'ip'},
			$p->{'port'},
			$p->{'network'},
			$p->{'users'},
			$p->{'songs'},
			$p->{'gigs'},
			$p->{'host'}
		   );
	
		if ($network) {
			if (not $p_network) {
				print "Error: Something wicked happened. Maybe ".
					          "napigator is down\n";
				exit(1);
			}
			next unless $p_network eq $network; 
		}
		
		printf "Trying ".GREEN.$p_host.RESET." on port ".
			BLUE.$p_port.RESET." ($p_ip:$p_port)...\n";
			
		if (not $meta) {
			$nap = MP3::Napster->new(-server => $p_ip.":".$p_port);
			if (!defined ($nap)){
				printf STDERR "Error: Server unreachable\n";
				next;
			}
		}
		else {
			$nap = MP3::Napster->new(-meta => $p);
		}
	
		if (not $nap->login($user, $pass, LINK_UNKNOWN, $local_port) or $@) {
			printf BLUE."Couldn't connect to $p_ip:$p_port: ".RED.$nap->error.
			RESET."\n";
		}
		else {
			$nap->download_dir($down_dir);
			printf UNDERLINE.GREEN."Success: Logged in with nickname ".
			UNDERLINE.BLUE.$nap->nickname."!".RESET."\n";
			last;
		}
	}
	return $nap;
}																	      

sub login_to_server {
	my ($server, $local_port, $user, $pass, $dir) = @_;
	my ($ip, $port) = split /:/, $server;
	$port ||= 8888;

	my $nap = MP3::Napster->new(-server => $ip.":".$port);
	if (not $nap->login($user, $pass, LINK_UNKNOWN, $local_port) or $@) {
		printf BLUE."Couldn't connect to $ip:$port: ".RED.$nap->error.
 			   RESET."\n";
	}
	return $nap;
}
									
#------------------------------
# Register instead of loggin in
# 95% redundancy with log_in()
#------------------------------

sub register_account {
	my ($meta, $network, @peers) = @_;
	my $nap;

	my ($user, $pass, $link) = (get_user(), get_pass(), get_link());
	my %details = (link  => $link);
	my $email;

        for my $p (@peers) {
		my (	$p_ip,
         	    $p_port,
                $p_network,
       		    $p_users,
			    $p_songs,    
	            $p_gigs,
	            $p_host,
		   ) =
	           ($p->{'ip'},
			    $p->{'port'},
	            $p->{'network'},
	            $p->{'users'},
			    $p->{'songs'},
	            $p->{'gigs'},
			    $p->{'host'}
		   );
		   
		if (defined ($network)) {
			next unless $p_network eq $network;
		}
		else {		
			printf RED."Error: You must specify a network for registering.".
			       RESET."\n";
			exit(1);
		}
			
		printf "Trying registration at ".GREEN.$p_host.RESET." on port ".
			BLUE.$p_port.RESET." ($p_ip:$p_port)...\n";
			
		if (not $meta) {
			$nap = MP3::Napster->new(-server => $p_ip.":".$p_port);
			if (!defined ($nap)){
				printf STDERR "Error: Server unreachable\n";
				next;
			}
		}
		
		if (defined($nap)) {
			$email = $nap->register($user, $pass, \%details);
			if (not $nap->error) {
				printf BLUE."Registration successfull since there's no ".
						    "error message from the server.";
				if (not $email) {
					printf BLUE."\nBut no email address either, ".
						    	" so hope for the best!".
					       RESET."\n";
				}
				else {
					printf BLUE."$email is the email address assigned to you ".
						    	"by the server.".RESET."\n";
				}
				return 1;
			}
			
			else {
				printf RED."Registration on $p_host failed: ".
				       $nap->error.RESET."\n";
				next;
			}
		}	
	}
}

#-------------
# get username
#-------------
sub get_user {
	printf "give username: ";
	chomp (my $user = <STDIN>);
	return $user;
}

#-------------
# get password
#-------------
sub get_pass {
	printf "give password: ";
	# make concole non-echoing
	system "stty -echo </dev/tty" unless $ENV{EMACS};
	chomp (my $pass = <STDIN>);
	system "stty echo </dev/tty" unless $ENV{EMACS};
	printf "\n";
	return $pass;
}

#--------------
# get linespeed
#--------------
sub get_link {
	my $link;
	while() {
		printf "give linespeed: ";
		chomp ($link = <STDIN>);
		$link = uc $link;
		$link =~ s/.*/LINK_$&/;
		if(not exists $LSPEED{$link}) {
			print "\n".$link." is not a valid linespeed\n".
				  "Use one of UNKOWN, 14K, 28K, 33K, 56K, 64K, 128K".
				  ", Cable, DSL, T1 or T3.\n";
		}
		else { last; }
	}
	return $LSPEED{$link};
}
1;
