# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap networking module $Revision: 1.4 $
# last change made $Date: 2001/05/18 19:25:41 $
# Thanks go to Travis Whitton from whom
# I borrowed a little bit of code

package AutoLib::Net;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(&get_servers);
	%EXPORT_TAGS  = ();

	}
use Socket;
use strict;
use vars     @EXPORT_OK;

my  $Cache_Dir = $ENV{HOME}."/.autonap";
mkdir $Cache_Dir, 0755 if not -e $Cache_Dir;

sub get_servers {

	my ($peer, $port, $cached) = @_;
	my @peers;
	
	# try a connection tp $peer:$port unless $cached
	my ($proto, $in, $sin, $msg);
	if (not $cached) {
		$proto = getprotobyname('tcp');
		$in = inet_aton($peer);
		$sin = sockaddr_in ($port, inet_aton($peer));
		$msg = "GET /servers.php\n\n";
		socket (S, PF_INET, SOCK_STREAM, $proto);
		connect (S, $sin) 
			or warn "Warning: $peer:$port not reachable: $!";
		syswrite (S, $msg, length($msg));

		open CACHE, ">$Cache_Dir/list"
			or warn "Warning: Could not create cache file in $Cache_Dir: $!";
	}

    if (not $in) {
		print "Warning: Couldn't properly resolve www.napigator.com.\n".
			  "Will therefore try to use a cached serverlist.\n" 
			  	unless $cached;
		open CACHE, "<$Cache_Dir/list"
			or die "Error: Could not even find one, so exiting.\n";
	}
												  
	my $HANDLE;
	if ($in) { $HANDLE = \*S }
	else     { $HANDLE = \*CACHE }
	
	while (<$HANDLE>) {
		
		print CACHE $_ if $HANDLE == \*S;
		
		my ($ip, $port, $network, $users, $songs, $gigs, $host) = split;
		my $server = { ip      => $ip,
			     	   port    => $port,
			           network => $network,
					   users   => $users,
			           songs   => $songs,
			           gigs    => $gigs,
			           host    => $host };
		push @peers, $server;

	}
	
	close CACHE;
	return @peers;

}

1;
