# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap server module $Revision: 1.8 $
# last change made $Date: 2001/05/15 06:07:39 $

package AutoLib::Server;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(&start_server &read_by_char &read_by_line);
	%EXPORT_TAGS  = ();

	}

use strict;
use vars     qw(@EXPORT_OK);

use IO::Socket;
use Net::hostent;
use MP3::Napster;
use Term::ReadKey;
use Term::ANSIColor       qw(:constants);
use AutoLib::CommandProcessor;
use AutoLib::Shell        qw(&set_output_handle);
use AutoLib::Callbacks    qw(&set_CLB_output_handle &print_MOTD);

my $FILE;
my $client;
		
sub logmsg { print "$0 $$: @_ at ", scalar localtime, "\n" }

sub start_server {

	my ($port, $nap, $f) = @_;
	$FILE = $f;
	
	my $server = IO::Socket::INET->new(Proto     => 'tcp',
					   LocalPort => $port,
					   Listen    => SOMAXCONN,
					   Reuse     => 1) 
		or die "Error: $!";
												
	logmsg "server started on port $port";
	
	while ($client = $server->accept()) {

      		$client->autoflush(1);

		set_output_handle($client);
		set_CLB_output_handle($client);

		my $hostinfo = gethostbyaddr($client->peeraddr);
		my $ip	     = $client->peerhost;	
		
		printf "[Connect from %s (%s) at %s]\n", $hostinfo->name, $ip 
				|| $client->peerhost, scalar localtime();

		if (not _check_access_permission($ip)) {
			print $client 
				RED.BOLD."You're not granted to access this service, sorry.\n";
			print $client "Therefore, I will no longer listen to you.".
						  RESET"\n";
			print $client "EOL\n";
			next;
		}
		
		while (my $msg = <$client>) {

			_from_whom($hostinfo, $ip, $msg);

			if ($msg =~ /^exit/) {
				print $client RESET."\n\n";
				$client->shutdown(1);
				next;
			}
			my @ret = auto_nap_shell("junk", $msg, $client);
			print $client "EOL\n";

		}
	}
}

sub read_by_char {
	print $client "\nSTART_PAGING\n";
	my $key;
	sysread ($client, $key, 1);
	return $key; 
}

sub read_by_line {
	print $client "\nLINE_PLEASE\n";
	my $line;
	while (<$client>) {
		$line = $_;
		last;
	}
	return $line;
}

sub _from_whom {
	my ($hostinfo, $ip, $what) = @_;
	printf "*from %s (%s)*: %s", $hostinfo->name, $ip, $what;
}

#---------------------------
# check whether client ip is 
# in access-record
#---------------------------
sub _check_access_permission {
	my $ip = shift;
	my $access = 0;
	my @granted;

	open (CONF, $FILE) or 
		die "Error: I need to have a configfile ".
		    "when using -D for checking the ip-range:\n$!";
	
		while(<CONF>) {
			chop;
			if (/^dmn_access/) {
				my (undef, $s) = split /=/;
				$s =~ s/\./\\./g;
				$s =~ s/\*/\\d+/g;
				@granted = split / /, $s;
			}
		}

		for my $p (@granted) {
			print "matching $p against $ip...";
			if ($ip =~ /$p/) {
				$access = 1; 
				print "success\n";
				last;
			}
			else {
				print "no match\n";
			}
		}
		return $access;
}

			
	
1;
