# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap shell module $Revision: 1.33 $
# last change made $Date: 2001/05/22 17:40:39 $

import MP3::Napster;

package AutoLib::Shell;

BEGIN {
	use Exporter   ();
	use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION = 1.0;

	@ISA          = qw(Exporter);
	@EXPORT       = qw(&make_global &set_output_handle
			   		   &cmd_check &cmd_help &cmd_search &cmd_get 
	                   &cmd_downs &cmd_ddown &cmd_ups &cmd_dup
					   &cmd_quit &cmd_stream &cmd_browse &cmd_last 
					   &cmd_blast &cmd_ping &cmd_whois &cmd_rsearch 
					   &cmd_raw
					   &display_results
					   &len_to_secs
					   %LSPEED);
	%EXPORT_TAGS  = ( );

	}

use AutoLib::Callbacks;
use AutoLib::Cache;
use AutoLib::Server qw(&read_by_char &read_by_line);
use MP3::Napster::User;
use IO::File;
use Term::ReadKey;
use Term::ANSIColor qw (:constants);
use strict;
use vars     @EXPORT_OK;


# some globs
our ($NAP, $DIR, $PLAYER, $SCR_WIDTH, $SCR_HEIGHT, @RND_PATTERN);
our ($OUT, $IN);
our $PID;
our %LSPEED = (
		LINK_UNKNOWN => 1,
		LINK_14K     => 2,
		LINK_28K     => 3,
		LINK_33K     => 4,
		LINK_56K     => 5,
		LINK_64K     => 6,
		LINK_128K    => 7,
		LINK_CABLE   => 8,
		LINK_DSL     => 9,
		LINK_T1      => 10,
		LINK_T3      => 11
	      );

#-------------------------------
# make global variables 
# accessible throughout Shell.pm
#-------------------------------
sub make_global {
	($NAP, $DIR, $PLAYER, $SCR_WIDTH, $SCR_HEIGHT, my $ref_rnd_pattern) = @_;
	@RND_PATTERN = @$ref_rnd_pattern;
}

sub set_output_handle {
	($OUT, $IN) = (shift, shift);
}

#-----------------------
# provide help commandos
#-----------------------
sub cmd_help {	

	my $arg = shift;
	my @ret;

	# remove leading and trailing whitespaces
	$arg =~ s/^\s+|\s+$//;
		
	if ($arg eq 'help' or not $arg) {
		@ret  = (
			GREEN.
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s\n".
			"%-8.8s%-8.8s%-8.8s%-8.8s\n",
			"check", "blast","browse", "ddown", 
			"dup", "exit", "downs", "get", 
			"help", "last", "ping", "quit", 
		    "rsearch", "search", "stream", "ups",
			"whois",			
			);
	}
	elsif ($arg eq 'blast') {
		@ret = GREEN."blast <num> - just like 'last' but".
		             " accessing browse history";
	}
		
	elsif ($arg eq 'browse') {
		@ret = GREEN."browse <user> - show list of <user>'s".
			     " shared songs";
	}
	elsif ($arg eq 'check') {
		@ret = GREEN."check <song_num> - show info on given song".
		" more detailed";
	}
	elsif ($arg eq 'ddown') { 
		@ret = (
		       GREEN."%-6.6s%-11.11s - %-s\n%-6.6s%-11.11s -".
		       " %-s\n%-6.6s%-11.11s - %-s",
		       "ddown", "<song_num>", 
		       "deletes download of given song",
		       " ",     "<song_list>",
		       "deletes download of songs 'n-m' ('all' for all)",
		       " ",     "<song_enum>",
		       "deletes download of songs 'n,m,o...'"
		       );
	}
	elsif ($arg eq 'downs') {
		@ret = GREEN."downs - info on currently running downloads";
	}
	elsif ($arg eq 'dup') {
		@ret = GREEN."dup <song_num> - delete upload of given song";
	}
	elsif ($arg eq 'exit') {
		@ret = GREEN."exit - in interactive mode alias for 'quit'\n".
		             "       in daemon mode terminates only the client";
	}
	elsif ($arg eq 'get') {
		@ret = (
			GREEN.
			"%-4.4s%-11.11s - %-s\n%-4.4s%-11.11s - ".
			"%-s\n%-4.4s%-11.11s - %-s",
			"get", "<song_num>",
			"tries to download given song",
			" ",   "<song_list>",
			"tries to download song 'n-m'",
			" ",   "<song_enum>",
			"tries to download songs 'n,m,o...'"
			);
	}
	elsif ($arg eq 'rsearch') {
		@ret = GREEN."rsearch [speed] - performs a random search".
			                          " with at least [speed]\n".
		         	 "                  [speed] is optional and".
			     	 				  " defaults to T1";
	}
	elsif ($arg eq 'search') {
		@ret = (
			GREEN."%-7.7s%-28.28s - %-s\n%-7.7s%-31.31s%-s",
			"search", "<pattern>[||<min_linespeed>]",
			"searches for files containing <pattern>",
			" ", " ",
			"unfortunately nap-servers don't yet support regexps"
			);
	}
	elsif ($arg eq 'stream') {
		@ret = GREEN."stream <song_num> - get and listen".
			        				    " to song at one time";
	}
	elsif ($arg eq 'last') {
		@ret = GREEN."last [<num>] - display last search's result\n".
		             "               if <num> is given, the <num>".
			     		             "th search's result\n".
			     	 "               is dislayed. 'last 0' also". 
			     		             "returns last search";
	}
	elsif ($arg eq 'ping') {
		@ret = GREEN.
		"ping <user> - ping user and return the time of the pong-reply";
	}
	elsif ($arg eq 'quit') {
		@ret = (
			GREEN."%-6.6s - %-s",
			"quit", "haha, have a guess"
			);
	}
	elsif ($arg eq 'whois') {
		@ret = GREEN.
		"whois <user> - retrieve some information on <user>";
	}
	elsif ($arg eq 'ups') {
		@ret = GREEN."ups - info on currently running uploads";
	}
	else {
		my $l = length($arg);
		$l++;
		@ret = (
			GREEN."%-".$l.".".$l."s - %-s\n%-".$l.".".$l."s   %-s",
		        $arg, "unknown command",
			" ", "try 'help' for info on available commands"
			);
	}
	
	return @ret;
}

#-------------------------
# provides search function
#-------------------------
sub cmd_search { 
	my $p = shift;

	# remove leading and trailing whitespaces
	# just in case...
        $p =~ s/^\s+|\s+$//;
	
	my ($artist, $linespeed) = split /\|\|/, $p;
	
	if (defined($linespeed)) { $linespeed =~ s/(.*)/LINK_$1/ }
	else { $linespeed = "LINK_DSL" }
	
	if (not $LSPEED{$linespeed}) {
		printf $OUT BLUE."-----\n".
		            RED."Wrong linespeed".
			    BLUE."\n-----".RESET."\n";
		return my @found;
	}

	my @found = $NAP->search(artist=>$artist,
		                 linespeed=>$linespeed);	
	@found = sort { $b->{'link'} <=> $a->{'link'} } @found;
	# unless search retrieved no results, cache
	cache_search(@found) unless @found == 0;
	display_results(@found);
	return @found;
}

#-----------------------------
# same as search but randomly
# this conventiently will just
# generate a pattern and pass
# it to cmd_search()
#-----------------------------
sub cmd_rsearch {
	my $speed = shift || "T1";
	$speed =~ s/\s+//g;

	# get a random pattern
	my $len = scalar(@RND_PATTERN);
	my $which = sprintf "%d", rand $len;
	my $pattern = $RND_PATTERN[$which];
	return cmd_search($pattern."||".$speed);
}
																			

	
	

#--------------------------------------
# browse another user's song-repository
#--------------------------------------
sub cmd_browse {
	my $user = shift;
	
	# remove leading whitespaces
	$user =~ s/^\s+//;
		
	if (not $user) {
		printf $OUT BLUE."-----\n";
		printf $OUT RED."%-s", 
		       "You should give a user's name to browse his archive"; 
		printf $OUT BLUE."\n-----".RESET."\n";
		return;
	}
	
	my @found = $NAP->browse($user);
	
	if ($NAP->error) { 
		printf $OUT RED.$user.": ".$NAP->error." (Warning).".
		RESET."\n"; 
	}
	
	# unless browse retrieved no results, cache
	unless (@found == 0) {
		cache_browse(@found) unless @found == 0;
		display_results(@found);
		return @found;
	}
	else  {
		printf $OUT BLUE."-----\n";
		printf $OUT RED.
			"Either the user shares no songs ".
			"or connection to him timed out.";
		printf $OUT BLUE."\n-----".RESET."\n";
		return undef;
	}
}

#-------------
# pings a user
#-------------
sub cmd_ping {
	my $user = shift;

	# remove leading whitespaces
        $user =~ s/^\s+//;
	
	if (not $user) {
		printf $OUT BLUE."-----\n";
	        printf $OUT RED."%-s",
                           "You should give a user's name to send the ping to";
	        printf $OUT BLUE."\n-----".RESET."\n";
	        return 0;
	}
	
	my $st = time;
	if ($NAP->ping($user)) {
		my $et = time;
                printf $OUT BLUE."-----\n".
		       GREEN."Received response within ".($et - $st)." secs.".
		       BLUE."\n-----".RESET."\n";
	}
	else {
		printf $OUT BLUE."-----\n".
		       RED."No PONG reveived.".
		       BLUE."\n-----".RESET."\n";
	}							       
}

#------------------------------
# displays last search's result
#------------------------------
sub cmd_last {
	my $arg = shift;

	# remove leading and trailing whitespaces
        $arg =~ s/^\s+|\s+$//;
		
	if ($arg =~ /^\d+$/ and $arg != 0 and $arg <= num_of_search()) {
		$arg--;
		my @f = get_from_cache("search", $arg);
		display_results(@f);
		return @f;
	}
	elsif ($arg eq 'last' or ($arg =~ /^\d+$/ and $arg == 0)) {
		if (num_of_search() > 0) {
			my @f = get_from_cache("search", -1);
			display_results(@f);
			return @f;
		}
		else {
			printf $OUT BLUE."-----\n";
			printf $OUT RED."No search done yet.";
			printf $OUT BLUE."\n-----".RESET."\n";
			return undef;
		}
	}
	elsif ($arg =~ /^\d+$/ and $arg > num_of_search()) {
		printf $OUT BLUE."-----\n";
		printf $OUT RED.
		"There only have been ".num_of_search()." searches so far.";
		printf $OUT BLUE."\n-----".RESET."\n";
		return undef;
	}
	else {
		printf $OUT BLUE."-----\n";
		printf $OUT RED.
		"Argument of 'last' either must be numeric or none at all.";
		printf $OUT BLUE."\n-----".RESET."\n";
		return undef;
	}
}

#-----------------------------
# Display last browse's result
#-----------------------------
sub cmd_blast {
	my $arg = shift;

	# remove leading whitespaces
        $arg =~ s/^\s+|\s+$//;
		
	if ($arg =~ /^\d+$/ and $arg != 0 and $arg <= num_of_browse()) {
		$arg--;
		my @f = get_from_cache("browse", $arg);
		display_results(@f);
		return @f;
	}
	elsif ($arg eq 'blast' or ($arg =~ /^\d+$/ and $arg == 0)) {
		if (num_of_browse() > 0) {
			my @f = get_from_cache("browse", -1);
			display_results(@f);
			return @f;
		}
		else {
			printf $OUT BLUE."-----\n";
	   		printf $OUT RED."No browse done yet.";
			printf $OUT BLUE."\n-----".RESET."\n";
			return undef;
		}
	}
	elsif ($arg =~ /^\d+$/ and $arg > num_of_browse()) {
		printf $OUT BLUE."-----\n";
		printf $OUT RED."There only have been ".
			        num_of_browse()." browses so far.";
		printf $OUT BLUE."\n-----".RESET."\n";
		return undef;
	}
	else {
		printf $OUT BLUE."-----\n";
		printf $OUT RED.
		"Argument of 'blast' either must be numeric or none at all.";
		printf $OUT BLUE."\n-----".RESET."\n";
		return undef;
	}
}



#-----------------------------
# details about specified song
#-----------------------------
sub cmd_check {
	my ($arg, @found) = @_;
	my $song = undef;

	# remove leading and trailing whitespaces 
        $arg =~ s/^\s+|\s+$//;
	
	if ($arg =~ /^\d+$/ and ($arg <= @found or $arg >= 1)) {
		$song = $found[$arg - 1];
	}
	else {
		printf $OUT BLUE."-----\n";
		printf $OUT RED."No such item ($arg) to check";
		printf $OUT BLUE."\n-----".RESET."\n";
		return 0;
	}
	my ($title, $size, $bitrate, $freq, 
	    $owner, $length, $hash, $address, $link) =
		(
			$song->name,
			$song->size,
			$song->bitrate,
			$song->freq,
			$song->owner,
			$song->length,
			$song->hash,
			$song->address,
			$song->link
		);
		
	$size = $size / 1024 / 1024;
	$size = sprintf "%-4.2f", $size;
	
	my $sec = $length % 60;
	my $min = ($length - $sec) / 60;
	$length = $min.":".$sec;

	printf $OUT BLUE."-----\n";
	printf $OUT BLUE."Song:   ".GREEN.$title."\n";
	printf $OUT BLUE."Owner:  ".GREEN.$owner."\n";
	printf $OUT BLUE."Link:   ".GREEN.$link."\n";
	printf $OUT BLUE."Length: ".GREEN.$length." min\n";
	printf $OUT BLUE."Size:   ".GREEN.$size." mb\n";
	printf $OUT BLUE."Freq:   ".GREEN.$freq." Hz\n";
	printf $OUT BLUE."Bitr:   ".GREEN.$bitrate." kb/s\n";
	printf $OUT BLUE."MD5:    ".GREEN.$hash."\n";
	printf $OUT BLUE."Addr:   ".GREEN.$address;
	printf $OUT BLUE."\n-----".RESET."\n";
	return 1;
}

#------------------------------
# provides downloading function
#------------------------------
sub cmd_get {	
	my @songs;
	my ($arg, @found) = @_;
	
	# remove leading and trailing whitespaces 
	$arg =~ s/^\s+//;
	

	if ($arg =~ /^\d+$/) {
	
		if ($arg > @found or $arg < 0) {
			printf $OUT BLUE."-----\n".
				    RED."No such item ($arg) to get".
				    BLUE."\n-----".RESET."\n";
		}
	
		elsif ($arg =~ /^(\d{1,3},)+\d{1,3}$/) {
			@songs = split /,/, $arg; 
		}
		elsif ($arg =~ /^(\d)+-(\d)+$/) {
			my ($first, $last) = split /-/, $arg;
			for (my $i = 0; $i <= $last - $first; $i++) {
				$songs[$i] = $first + $i;
			}
		}
		elsif ($arg =~ /^\d+/) {
			if ($arg > 0) {
				push @songs, $arg;
			}
			else {
				printf $OUT BLUE."-----\n".
				    	    RED."Invalid song number ($arg)".
						    BLUE."\n-----".RESET."\n";
			}
		}

	}

	else
	{
		printf $OUT BLUE."-----\n";
		printf $OUT RED."%-s", 
			"$arg: Invalid arguments for 'get'";
		printf $OUT BLUE."\n-----".RESET."\n";
	}
	
	for my $s (@songs) {
		my $curSong = $found[$s - 1]->{'name'};
		my $handle = $DIR."/".$curSong;
		my $download;
		if ($found[$s - 1]->owner->ping) {
			$download = $NAP->download($found[$s - 1], $handle);
			$download->interval(2000) if (defined($download));
		}
		else {
			printf $OUT BLUE."-----\n".
			            BOLD.RED."%-s\n", 
				    "No PONG received for $curSong.".
				    BLUE."\n-----".RESET."\n";
		}
	}			
	
}

#------------------------------
# delete a running download job
#------------------------------
sub cmd_ddown {
	my $no = shift;

	# remove leading whitespaces from $arg
	$no =~ s/^\s+//;

	if ($no !~ /^\d+$/) {
		printf $OUT BLUE."-----\n".
			        RED."Argument of ddown must be numeric".
			   	    BLUE."\n-----".RESET."\n";
		return;
	}
	
	my @transfers = $NAP->downloads;
	if ($no < 1 or $no > @transfers) {
		printf $OUT BLUE."-----\n".
		       RED."No such item ($no)".
		       BLUE."\n-----".RESET."\n";
	}
	else {
		$transfers[$no - 1]->abort;
	}
}

#----------------------------
# delete a running upload job
#----------------------------
sub cmd_dup {
	my $no = shift;

	# remove leading whitespaces from $arg
	$no =~ s/^\s+//;

	if ($no !~ /^\d+$/) {
		printf $OUT BLUE."-----\n".
					RED."Argument of dup must be numeric".
					BLUE."\n-----".RESET."\n";
		return;
	}
	
	my @transfers = $NAP->uploads;

	if ($no < 1 or $no > @transfers) {
		printf $OUT BLUE."-----\n".
					RED."No such item ($no)".
					BLUE."\n-----".RESET."\n";
	}
	else {
		$transfers[$no - 1]->abort;
	}
}


#---------------------------------------
# returns infos on running download jobs
#---------------------------------------
sub cmd_downs {

	my $ord = 1;
	printf $OUT BLUE."-----\n";
	for my $song ($NAP->downloads) {
		my ($text, $transferred, $size, $perc);
		$text        = $song->asString;
		$transferred = $song->transferred;
		$size        = $song->size;
		if ($size) {
			$perc = $transferred / $size * 100; 
		}
		
		if (not $perc) {
			printf $OUT GREEN."%-4s - %-s\n%-s\n",
			       "($ord)", $song->asString,
			       "Status: ".$song->status;
		}
		else {
			printf $OUT  GREEN.
				"%-4s - %-s\n%-d / %-d = %-2.2f%%\n%-s\n", 
		        	"($ord)", $song->asString,
			 	$transferred, $size, $perc,
				"Status: ".$song->status;
		}
		$ord++;
	}
	printf $OUT BLUE."-----".RESET."\n";
}

sub cmd_ups {

	my $ord = 1;
	printf $OUT BLUE."-----\n";
	for my $song ($NAP->uploads) {
		my ($text, $transferred, $size, $perc);
		$text        = $song->asString;
		$transferred = $song->transferred;
		$size        = $song->size;
		if ($size) {
			$perc = $transferred / $size * 100;
		}
		
		if (not $perc) {
			printf $OUT GREEN."%-4s - %-s\n%-s\n",
		  		   			  "($ord)", $song->asString,
							  "Status: ".$song->status;
		}
		else {
			printf $OUT  GREEN."%-4s - %-s\n%-d / %-d = %-2.2f%%\n%-s\n",
							   "($ord)", $song->asString,
								$transferred, $size, $perc,
							   "Status: ".$song->status;
		}
		$ord++;		
	}
	printf $OUT BLUE."-----".RESET."\n";
}


#---------------------------------------------------------------------
# tees the song into a file and an mp3player specified in a configfile
#---------------------------------------------------------------------
sub cmd_stream {
	my ($arg, @found) = @_;
	my $song;
	my $curSong;

	# remove leading whitespaces
	$arg =~ s/^\s+//;
		
	if ($arg =~ /^\d+$/ and $arg > 0) {

		$song = $found[$arg - 1];
		$curSong = $song->name;
		my $handle = IO::File->
			new("| tee $DIR/".quotemeta($curSong)." | $PLAYER");
							
		my $download;
		if ($song->owner->ping) {
			$download = $NAP->download($song, $handle);
			$download->interval(2000) 
				if (defined($download));
		}
		else {
			printf $OUT BLUE."-----\n";
			printf $OUT BOLD.RED."%-s\n", 
				"No PONG received for $curSong.";
			printf $OUT BLUE."\n-----".RESET."\n";
		}
	}
	else {
		printf $OUT BLUE."-----\n".
		       RED."Invalid song number".
		       BLUE."\n-----".RESET."\n";
	}	
}

#---------------------
# retrieve information
# on given user
#---------------------
sub cmd_whois {
	my $user = shift;
	$user =~ s/^\s+//;
	
	my ($u_name, $u_link, $u_share, $u_chans, $u_ups, $u_downs, 
	    $u_level, $u_status, $u_ltime, $u_client);

	my $u = $NAP->whois($user);

	if (not $u) {
		printf $OUT BLUE."-----\n".
			    RED."No information available on $user".
			    BLUE."\n-----".RESET."\n";
	}
	else {
		
		$u_status = $u->status;
	
		if ($u_status eq 'Active') {
			$u_name   = $u->name;
			$u_link   = $u->link;
			$u_share  = $u->sharing;
			my @u_chans  = $u->channels; 
			$u_chans = join ", ", @u_chans;
			$u_ups    = $u->uploads;
			$u_downs  = $u->downloads;
			$u_level  = $u->level;
			$u_ltime  = format_login_time($u->time);
			$u_client = $u->client;
		
			printf $OUT BLUE."-----\n";
			printf $OUT BLUE."Name:       ".GREEN.$u_name."\n"; 
			printf $OUT BLUE."Speed:      ".GREEN.$u_link."\n";
			printf $OUT BLUE."Shares:     ".GREEN.$u_share."\n";
			printf $OUT BLUE."Channels:   ".GREEN.$u_chans."\n";
			printf $OUT BLUE."Uploads:    ".GREEN.$u_ups."\n";
			printf $OUT BLUE."Downloads:  ".GREEN.$u_downs."\n";
			printf $OUT BLUE."Level:      ".GREEN.$u_level."\n";
			printf $OUT BLUE."Status:     ".GREEN.$u_status."\n";
			printf $OUT BLUE."Login time: ".
				    GREEN.$u_ltime." (h:m:s)\n";
			printf $OUT BLUE."Client:     ".GREEN.$u_client;
			printf $OUT BLUE."\n-----".RESET."\n";
		}
		else {
			printf $OUT BLUE."-----\n".
				    RED."$user is currently offline".
				    BLUE."\n-----".RESET."\n";
		}
	}
}
		
#----------------------------------------
# quit connection and destroy objects
# MP3::Napster kindly handles that for us
# make sure to reset terminal
#----------------------------------------
sub cmd_quit {
	ReadMode 0;
	printf $OUT UNDERLINE.GREEN."Thanks for using AutoNap!";
	printf $OUT RESET."\n";
	exit(0);
}

#------------------------------
# sends a raw message to server
#------------------------------
sub cmd_raw {
	my $msg = shift;
	print $msg;
	$NAP->send($msg, "AutoNap");
}

#---------------------------------
# Print results formatted properly
#---------------------------------
sub display_results {
	
	my $key;

	printf $OUT GREEN."\n%-4s".BLUE."|".GREEN."%-46.46s".BLUE."|".
		    GREEN."%-8s".BLUE."|".GREEN."%-3s".BLUE."|".
		    GREEN."%-6s".BLUE."|".GREEN."%-8s\n", 
	       "No.", "Song" , "Size", "bit", "len", "speed";
	printf $OUT BLUE;
	printf $OUT "-" x 4 ; printf $OUT "|"; 
	printf $OUT "-" x 46; printf $OUT "|"; 
	printf $OUT "-" x 8 ; printf $OUT "|";
	printf $OUT "-" x 3 ; printf $OUT "|";
	printf $OUT "-" x 6 ; printf $OUT "|";
	printf $OUT "-" x 8 ; printf $OUT RESET."\n";
	
	ReadMode 4;	

	my @found = @_;
	my $c = 1;
	for (@found) {
		
		my (
		$name, 
		$size,
		$bitrate,
		$seconds, $minutes, $length, 
		$speed
		);
	
		$name = substr $_->{'name'}, 0, -4;
		$seconds = $_->{'length'} % 60;
		$minutes = ($_->{'length'} - $seconds) / 60;
		$length = $minutes;
		if ($seconds > 9) {
			$length = $length.":".$seconds;
		}
		else {
			$length = $length.":0".$seconds;
		}
		$size = $_->{'size'} / 1024 / 1024;
		$bitrate = $_->{'bitrate'};
		$speed = LineSpeed($_->{'link'});
	
		printf $OUT "%-4d".BLUE."|".RESET."%-46.46s".BLUE."|".
			    RESET."%-4.1f%3.2s ".BLUE."|".RESET."%-3d".
			    BLUE."|".RESET."%-6.6s".BLUE."|".RESET."%-8s\n",
			$c,
			$name,
			$size,"mb",
			$bitrate,
			$length,
			$speed;
			$c++;
	 	
		if ($c % ($SCR_HEIGHT - 2) == 0) {  
			
			LABEL_G:
			
			print $OUT GREEN, "-- any key to continue (q to quit listing) --",
					   RESET;

			# we are not in daemon mode
			if ($OUT == \*STDOUT) {
				while (not defined($key = ReadKey(-1))) {
	    				# wait
				}
				print $OUT "\n";
			}
			
			# we are in daemon mode hence needing a different capture of input
			elsif ($OUT != \*STDOUT) {
				while (not defined($key = read_by_char())) {
 					# wait
				}
			}

			#print "\n";
			if ($key eq 'g' or $key eq 'x') {
				ReadMode 0;
				my $num;
				if ($OUT == \*STDOUT) {
					printf $OUT BLUE."Give song number: ";
					$num = <STDIN>;
				}
				elsif ($OUT != \*STDOUT) {
					$num = read_by_line();
				}
				printf $OUT RESET."\n";
				last if $num eq "\n";
				cmd_get($num, @found);
				last if $key eq 'x';
				ReadMode 4;
				goto LABEL_G;
			}
			if ($key eq 'q') {
				last;
			}
   		}
	}
	ReadMode 0;
	return 1;
}

#----------------------------------
# conversion form line code to text
#----------------------------------
sub LineSpeed {

        my $l = shift;
        my %speedtable = (
                0  => '???',
                1  => '14K',
                2  => '28K',
                3  => '33K',
                4  => '56K',
                5  => '64K',
                6  => '128K',
                7  => 'Cable',
                8  => 'DSL',
                9  => 'T1',
                10 => 'T3',
	                );
        return $speedtable{$l};

}

#----------------------------
# convert song length to secs
#----------------------------
sub len_to_secs {
        my $t = shift;
	my ($min, $sec) = split /:/, $t;
	return $min * 60 + $sec;
}

#----------------------
# nicely format secs to 
# hour:min:sec
#----------------------
sub format_login_time {
	my $secs = shift;
	my $remain_secs = $secs % 60;
	my $mins = ($secs - $remain_secs) / 60;
	my $remain_mins = $mins % 60;
	my $hours = ($mins - $remain_mins) / 60;
	return sprintf "%0d:%02.2d:%02.2d", $hours, $remain_mins, $remain_secs;
}

1;
