#!perl

# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# AutoNap $Revision: 1.21 $ 
# last change made $Date: 2001/05/25 05:44:00 $

#use lib 

require 5.6.0;

use strict;
use diagnostics;
use Term::ReadKey;
use Term::ANSIColor qw (:constants);
use MP3::Napster;
use AutoLib::Net;
use AutoLib::Login;
use AutoLib::CommandProcessor qw(&auto_nap_shell &get_user_input $term);
use AutoLib::Shell;
use AutoLib::Callbacks qw(
				&set_CLB_output_handle
			  	&CLB_message_of_day
			  	&CLB_transfer_progress
			  	&CLB_transfer_start
			  	&CLB_transfer_end
			  	&CLB_transfer_status
			  	&print_MOTD
			);
use AutoLib::Conffile  qw(
				$ACC_NETWORK         	$ACC_USER               
				$ACC_PASS            	$ACC_DOWNLOAD_DIR
				$MISC_PLAYER
       	 		$DOWN_MIN_LINESPEED  	$DOWN_MIN_BITRATE       
				$DOWN_MAX_SIMULTANEOUS  $DOWN_TIMEOUT        
				$DOWN_MIN_FILE_LENGTH	$SEARCH_PATTERN
       			$RND                 	@RND_EXCLUDE_PATTERN	
				&parse_configfile
						);
use AutoLib::Cache      qw(
				@CACHE_SEARCH @CACHE_BROWSE
				&cache_search &cache_browse
		        );
#use AutoLib::Trap;
use AutoLib::Server;

$0 = "AutoNap 0.5.1";

print RESET;

my 	(
	%OPT, %TRASH,
	$term, $prompt, $OUT,
	$key, $value,
	$nap,
	@found);

#----------------------------
# Parse commandline arguments 
# I don't know what's wrong
# with Getopt::Std and 
# Getopt::Long, but these two
# do not suffice
#----------------------------

my $cmd_length = @ARGV;
for (my $i = 0; $i < $cmd_length; $i++) {

	# it is an option taking an argument
	if ($ARGV[$i] =~ /^-[bdfFglnpPrsSu]$/) {
		$OPT{$ARGV[$i]} = $ARGV[$i+1] unless not $ARGV[$i+1] 
									          or $ARGV[$i+1] =~ /^-/;
	}
	
	# it is the -x option
	elsif ($ARGV[$i] =~ /^-x$/) {  
		# go to next element
		my $pos = $i + 1;  
		my $x_arg;
		# loop until it is again an option
		while (defined($ARGV[$pos]) and  $ARGV[$pos] !~ /^-.$/) {  
			$x_arg .= $ARGV[$pos]." ";
			$pos++;
		}
		$OPT{$ARGV[$i]} = $x_arg;
	}
	
	# it is a boolean option requiring no argument
	elsif ($ARGV[$i] =~ /^-[achmR]$/) { $OPT{$ARGV[$i]} = 1 }
	
	# it is the -D option
	elsif ($ARGV[$i] =~ /^-D$/) {
		if (not $ARGV[$i+1] or $ARGV[$i+1] =~ /^-/) { $OPT{$ARGV[$i]} = 1 }
		else { $OPT{$ARGV[$i]} = $ARGV[$i+1] }
	}
}

my $auto        = $OPT{'-a'};
my $h           = $OPT{'-h'};
my $file        = $OPT{'-f'};
my $server      = $OPT{'-S'};
my $meta        = $OPT{'-m'};
my $network     = $OPT{'-n'};
my $cache       = $OPT{'-c'};
my $port        = $OPT{'-g'};
my $user        = $OPT{'-u'};
my $pass        = $OPT{'-p'};
my $register    = $OPT{'-R'};
my $down_dir    = $OPT{'-d'};
my $player      = $OPT{'-P'} ||= "mpg123 -";
my $linespeed   = $OPT{'-l'};
my $bitrate     = $OPT{'-b'};
my $no_of_downs = '';
my $file_length = $OPT{'-F'};
my $song        = $OPT{'-s'};
my $daemon      = $OPT{'-D'};
my $repository  = $OPT{'-r'};
my @rnd_pattern = ();
my $not_these   = $OPT{'-x'} ||= 'britney spears::backstreet boys';

# besides help, this will gracefully ignore any illegal input

if ($h or (keys %OPT) == 0) { help() }

my @peers = get_servers("www.napigator.com", 80, $cache) unless $server;

#----------------------
# make some conversions
#----------------------

$player .= " -" unless $player =~ /\s-$/;
my @not_these = split $not_these, "::" if $not_these;

# fill @rnd_pattern with alphanums
for my $n (49..57) {
	push @rnd_pattern, chr($n); # [1-9]
}
for my $n (97..122) {
	push @rnd_pattern, chr($n); # [a-zA-Z]
}

# add some other things to it
push @rnd_pattern, ("and", "or", "not", "me", "you", "the", "is", "are", "we");
				
#----------------------------------
# Read options given in configfile.
# However, these wont override 
# commandline-switches.
#----------------------------------

if ($file) {
	parse_configfile($file);

	$network       = $ACC_NETWORK      			 unless $network;
	$user          = $ACC_USER        	 		 unless $user;
	$pass          = $ACC_PASS         			 unless $pass;
	$down_dir      = $ACC_DOWNLOAD_DIR 			 unless $down_dir;
	$player        = $MISC_PLAYER      			 unless $player eq 'mpg123 -';
	$linespeed     = "LINK_".$DOWN_MIN_LINESPEED unless $linespeed;
	$bitrate       = $DOWN_MIN_BITRATE			 unless $bitrate;
	$no_of_downs   = $DOWN_MAX_SIMULTANEOUS;
	$file_length   = len_to_secs($DOWN_MIN_FILE_LENGTH) unless $file_length;
	$song          = $SEARCH_PATTERN                    unless $song;
	@not_these     = @RND_EXCLUDE_PATTERN               unless @not_these;
}

# you want to register?

if ($register) {
	my $netw;
	if (not $network) {
		$netw ="whatever server we get first";
	}
	else {
		$netw = $network;
	}
	my $m = "Trying Registration on $netw: ";
	printf UNDERLINE.GREEN;
	printf "-" x (length($m) - 1);
	printf "\n".$m."\n";
	printf "-" x (length($m) - 1);
	printf RESET."\n";
	if (register_account($meta, $network, @peers)) { exit(0) }

	# damn!
	else {
		printf RED."Error: Couldn't register".RESET."\n";
		exit(1);
	}
}
					
	

# if users wants to be prompted,
# well, prompt him.

$user = get_user() if not $user or $user eq 'prompt_me';
$pass = get_pass() if not $pass or $pass eq 'prompt_me';

	    
#---------------------
# all went fine so far 
#---------------------
if ($server) { 
	$nap = login_to_server($server, $port, $user, $pass, $down_dir);
}
else { 
	$nap = log_in($network, $port, $meta, $user, $pass, $down_dir, @peers); 
}

if (not $nap or $nap->error) {
    	printf STDERR "Error: Couldn't create \$nap-object. ".
					  "That's out of your hands. (AutoNap)\n";
	exit(1);
}


if (exists $OPT{'-s'}) {
	@found = $nap->search(artist=>$song, 
			      		  bitrate=>"at least $bitrate", 
			      		  linespeed=>"at least $linespeed");
	display_results(@found);
}

#----------------------------------------
# If user wants non-interactive mode (-a)
# make this here:
# we will then ignore the rest here and
# turn to Auto.pm
# When ever he turns back, we exit
#----------------------------------------

if ($auto) {
	use AutoLib::Auto      qw(&auto_init);
	if (not $nap or not $file) {
		print "For non-interacting mode I need a conffigfile\n";
		exit(1);
	}
	auto_init( $nap, $down_dir, $file, @rnd_pattern );
	exit(0);
}

#----------------------------------------
# pass those variables unlikely to change
# to Shell.pm
# we need more of that later if we ever
# want to keep on maintining the code
#----------------------------------------

# make AutoNap fit the screen vertically (nifty)
my ($screen_width, $screen_height, undef, undef) = GetTerminalSize();

printf UNDERLINE.GREEN."Applying screensize of ".
	   $screen_width." x ".$screen_height."...".
       RESET."\n" unless $daemon;

$down_dir ||= "/tmp";

make_global($nap, $down_dir, $player, $screen_width, 
			$screen_height, \@rnd_pattern);
unless ($daemon) {
	set_output_handle(\*STDOUT);
	set_CLB_output_handle(\*STDOUT);
}

#---------------------
# user wants to share?
#---------------------

my $shared = 0;
if ($repository) {
	print BOLD.WHITE "Sharing songs...";
	$shared += $nap->share_dir($repository);
	print $shared." overall.".RESET."\n";
}
	

#--------------------------------
# add callbacks to napster object                  
#--------------------------------

$nap->callback(MOTD, \&CLB_message_of_day);
$nap->callback(TRANSFER_STARTED, \&CLB_transfer_start);
$nap->callback(TRANSFER_DONE, \&CLB_transfer_end);
$nap->callback(TRANSFER_STATUS, \&CLB_transfer_status);

	
if (not $daemon) {
	$nap->command_processor(\&auto_nap_shell);
	$nap->run;
}
else {
	if ($daemon > 1024) { start_server($daemon, $nap, $file) }
	elsif ($daemon == 1) { start_server(9876, $nap, $file) }
	else {
		print "Error: port must be in the valid range (1025 - 65535)";
		exit(1);
	}
}
	

#-------------------------
# display help und exit(0)
#-------------------------
sub help {

	my $help = <<HELP;

AutoNap $0 - an automatizable Napster-client
usage: AutoNap.pl [options] 
options:	
		-a              run automatically (thus no interaction)
		-d  <dir>       where to store downloads
		-D  [port]      run in daemon mode (thus accepting 
						client connections)
						if no port is given, defaults to 9876
		-f  <file>      read options from <file>
		-F  <length>    minimum filelength of song required
		-h     		 	print out this help
		-b  <bitrate>   set minimum bitrate to <bitrate>
		-l  <linespeed> set minimum linespeed to <linespeed>
		-m              specifies whether server (either passed 
						by -S switch or through configfile) is a 
						metaserver or not
		-n  <network>   try to connect to preferably this <network>
		-p  <pass>      login with password <pass>
		-P  <player>    specify mpg3-player unless you use 'mpg123'
		-r              repository of mp3s to share
						scans a directory recursively
		-R              register account, you should also give the -n 
						switch
		-x  <patterns>  random download but ignoring <patterns>            
		-s  <pattern>   search for pattern after startup
		-S  <server>    connect to <server>
		-u  <user>      login with username <user>
						only makes sense along with -p

		<file>			configfile that specifies AutoNap's behaviour
		<bitrate>       minimum bitrate of files to be returned 
						on a search
		<linespeed>     user's minimum linespeed 
		<pattern>		search for song files containing this pattern,
						multiword pattern embraces in quotes
		<network>       one of the opennap networks, such as 
						MusicCity or DJNap
		<server>        server in the form <address>:<port> where
					    <address> has to be an IP
		<user>			your Napster-username
		<pass>			password according to your username

HELP
	print $help;
	exit(0);
}


__END__

=head1 NAME

AutoNap.pl - an interacting and non-interacting Napster/OpenNap client

=head1 SYNOPSIS

AutoNap.pl [options] 

=head2 options

-h

-R [-n]

[-f] [-p -u -c -m -S -g -n -P -r -x -l -b -d -D]

[-f] [-g -p -s -r -a]

=head1 DESCRIPTION

AutoNap.pl is a program written in Perl which will connect to an OpenNap compatible network (Napster is one of those), search for given songs and download them.

It is intended either for interacting mode using a console color-interface or running unattendedly by following the directives given in a configfile. Starting with release 0.4.0 a third mode has been added which implements a client/server model communicating through tcp/ip.

Most of the initially announced features are finally there which does not mean all! Anything that is either missing or buggy should be reported to me. Bugs will usually be wiped out immediately while feature requests go to the Todo list or, in case it is an easy hack, into AutoNap itself.

=head1 ARGUMENTS

=over 4

=item -h 

Displays the help screen and exits.

=item -R

Register a new a accont. It is advisable to use -R only along with -N since AutoNap's serverlist is unsorted and thus you cannot predict which network it'll try to register you on.

=item -f <file>

Reads options from file. See the commented autonap.conf for details on how to setting up one.
You always need one when using '-a' or '-D'.

=item -a 

Do not start the interactive user's interface but instead run automatically according to the user's directives in the configfile. Output goes to a log-file.

Also needs a configfile ('-f').

=item -D [port]

Connects to a server as usual and then starts up a server accepting connections on [port]. Port need not necessarily be given as by default 9876 is assumed. For connecting to the daemon, use the supplied remote control AutoClient.pl. Paramters passed to AutoClient.pl just work as with telnet: 'AutoClient.pl 134.130.181.234 9876' for example. Telnet will also do but no directives are sent to the client to change into different reading modes therefore paging wont properly work. Just use AutoClient.

Also needs a configfile ('-f').

=item -u <user>

Tries to login to the network with <user>.

=item -p <pass>

Identifies with <pass> on login to a network.

=item -c

Will force AutoNap to not retrieve the serverlist from www.napigator.com. This is useful when you're sick of waiting for a (usually) slow connection to Napigator on startup or in case it's down. AutoNap will instead use $HOME/.autonap/list for servers. This files gets updated each time you successfully retrieve the list from the server.

=item -S <server>:<port>

Tries a login on <server> and <port>. <port> is mandatory when specifying a server. Since napster-servernames don't seem to have a valid DNS entry, AutoNap cannot resolve a servername to an IP. Therefore <server> needs to be an IP.

=item -n <network>

Logins to the specifies <network>. This only works when AutoNap was able to retrieve a serverlist from which it can deduce the network a server is belonging to. This means that -n doesn't make sense along with -S.

=item -g <local_port>

Specifies the local port for sharing files. It defaults to 6699. However, you may want to change that when running multiple instances of AutoNap. If you get the message that the listener could not be created, then go for this here.

=item -m 

Indicates that the given server is a meta-server which requires a slightly different approach on login.

=item -d <dir>

Store downloads into <dir>. The user running AutoNap naturally needs to have write permissions to this directory.

=item -b <bitrate>

Sets the default minimum bitrate to <bitrate>. That means that AutoNap will as default only find songs that have at least this bitrate. 

=item -l <linespeed>

Works just like the --bitrate option only that it affects the linespeed of songs to be found. This options can be overriden when being inside AutoNap by suffixing the searchpattern with ||<linespeed>. Valide values for <linespeed> are: 
T3, T1, DSL, Cable, 128K, 64K, 56K, 33K, 28K, 14K and UNKNOWN.

=item -P <player>

Specifies which mp3-player should be used for streaming. Only necessary to be specified when different from mpg123.

=item -r 

Toggles random mode, that is: tries to act as 'randomly' on search es possible. Intended for non-interact mode which isn't yet done.

=item -x <patterns>

Do random searches but none containing one of the <patterns>. <patterns> is a double-colon separated list.

=item -s <pattern>

Search for songs containing <pattern>. This switch is incompatible with -r and thus ignored when random is set.

=head1 EXAMPLES

This will try to register you on a MusicCity server. It will prompt you for desired username and password:

B<AutoNap.pl -R -n MusicCity>

Read options from /home/ethan/autonap.conf but only try MusicCity servers:

B<AutoNap.pl -f /home/ethan/autonap.conf -n MusicCity>

Try to login to any server with user/pass foo/bar and set mp3player to mp3blaster:

B<AutoNap.pl -u foo -p bar -P mp3blaster>

And so on, and so on. When you are smart, you should know yourself which options make sense with each other and which don't. ;-)

=head1 INTERACTIVE MODE

When using interactive mode (which is currently the only available one), you see yourself presented with the server's MOTD after login if there is any. You then can control AutoNap through the built-in commands which are listed to you when typing 'help'. Typing 'help <command>' gives you further advice on a particular command.

Some of the commands take arguments, some are without and some may take arguments. Commands without any further arguments are 'downs' and 'quit'. 'last' and 'help' can take arguments and for all other commands an argument must be supplied.

Prefixing a command with a '!' will invoke the following string to be interpreted as a system command. However, shell built-ins do not work, in order to execute these (things such as 'cd' etc.) you can do (for the bash):

B<!bash -c 'cd ..'>

As this is clumsy and hardly a win over switching to another shell, you can just start a new shell by typing:

B<!bash>

and then comfortably doing all your stuff there. B<exit> will return you to AutoNap.

=head1 NON-INTERACTIVE MODE

This mode is intended for those who want an mp3 leecher that can run as a cron-job for instance. Or those who want to fill their harddrive overnight without any own activity.

As AutoNap, once started with the -a option, can no longer be influenced (except by killing it), all options need to be passed to it in beforehand, preferably through the autonap.conf. For testing purpose, you can however trace what AutoNap is doing by invoking B<tail -f auto.log>. This is useful if you doubt that AutoNap will find any songs relating to the search pattern you specified. In this case you could interrupt AutoNap and start it once more with modified parameters.

Basically, you have to keep in mind that not even AutoNap can perform black magic and that it follows certain hard-coded rules when running non-interactively. It is helpful for you to know what it does and it which order.

First of all it logs on a network just as it does in interactive mode. Afterwards it has a closer look at the config-file and will read out the options given in the [rnd] and [auto] section. If it cannot find satisfactory information it will either assign some in its opinion reasonable defaults...or in worst case it will break. This will probably happen if you specified no values at all.

AutoNap's next step is assigning each search pattern to an element of an array. Thus it can be assured that later search results retrieved on different search patterns will not be merged. It will now perform several searches, depending on what you told him in the configfile. Assuming you gave him three search patterns and a bitrange of 128 and 160 kbits, AutoNap will do 3 * 2 = 6 searches. For each search pattern it will look for songs with at least 128 kbits and at least 160 kbits. It will merge search results from different bitrates per pattern und filter out any song that either is not in the range of the bitrates or that is not within the range of song-length (for instance 2:00 minutes and 5:00 minutes).

If you set B<ping=yes> in the conffile each song (better: its owner) is pinged and the roundtrip time together with the song put into a hash. After all pings have been done, all songs belonging to the respecitve search pattern are sorted by ping-reply.
Pinging takes a lot of time, you should be aware of that. If you don't want AutoNap to ping the owners, that's also fine with it. Then it'll just try to get a song one after another till a download has been succesfully finished.

AutoNap will never download more than one song relating to one search pattern. So when you specify B<beatles +help::beach boys +surfin +usa> it will at most get two songs and you do not have to worry that AutoNap pumps 'Help' by the Beatles a few times into your computer.

It behaves differently on random-searches. If you do not give it a limit (B<auto_max_download=3> for a maximum of three downloads) it will download as many songs as it will find and be able to download. Thus you can broaden your musical horizont by letting AutoNap run randomly while you sleep in the night. Next morning you should have a handful of probably new songs.

You'll find more detailed information in the L<autonap.conf(5)>.

=head1 DAEMON-MODE

Most importantly: NEVER LET THE DAEMON RUN AS ROOT!
I just did not do enough teste with the access-granting thing based on ips.

When invoking AutoNap.pl with the -D switch, AutoNap.pl will establish a connection with a napster-server, login and then fall silent. It'll listen for incoming connections on the port it has been started with.

Even on a local machine, it is a cool thing to let AutoNap run in daemon mode since AutoClient provides a convenient shell with command-history etc. The client can be left in two different ways either using 'quit' which will also shutdown the daemon or 'exit' which will just terminate the client and otherwise leaving the daemon untouched. Connections can be made later at any time.

As for now AutoNap's daemon is unthreaded which means it can only deal with one connection at a time. There is no packet-forwarding either, so any mp3-transfers will go to the server and not yet the client.

If you consider using AutoNap.pl mostly in daemon mode, please carefully read the bottom of this document as for bugs and limitations. It'll point out why it would be too crucial letting AutoNap run with uid=root.

=head1 PAGING

When browsing other user's song archive or retrieving results from a search, AutoNap wont scroll through to the bottom without you even catching a glimpse of the song. It will pause after (currently) 25 songs and ask you to press a key to go on scrolling or hitting 'q' to quit the list prematurely. There are two more keys you can press, 'g' and 'x'. 

'g' is useful when you just spied a song on the current page you badly need to have and you can't wait to scroll the list to the bottom or taking the pain on you to hit 'q' and then entering 'get <num>'. So it'll prompt you for this number, trying to initiate the download and return into paging mode. 'x' (short for eB<x>ecute and eB<x>it;-) does the same but wont return and instead fall back to the AutoNap shell.  

Both 'g' and 'x' wont affect 'last' in any way and prematurely leaving the list with 'q' will still cache the result, so you can rebrowse that at a later time.
In case you mistakenly hit 'x' or 'g' and you do not want to download any of the songs, just hit RETURN without any input and it'll savely make you leave the prompt without the need of downloading anything.

=head1 KNOWN BUGS

There is as to yet a little annoyance when using AutoNap in daemon-mode due to the nature of the callbacks. Therefore the MOTD-callback is currently disabled when using -D.

Furthermore, remotely executing shell-commands (with prefixing the input with '!') isn't correctly handled so output is on server-side. Since any commands are still executed, the daemon should NEVER run as root (uid=0) even though the B<dmn_access> line in the configfile should block the daemon against unwanted access.

Strange results occur when backgrounding the daemon using '&'. I haven't yet been up for a solution to that but probably will seek for a hard-coded way of backgrounding AutoNap. What I currently use for this purpose is screen which runs AutoNap in a virtual terminal that can be attached with Ctrl-a d and fetched back with 'screen -r'.

=head1 SEE ALSO

L<autonap.conf(5)> 

=head1 AUTHOR AND COPYRIGHT

Tassilo von Parseval
<Tassilo.Parseval@post.RWTH-Aachen.de>.

Copyright (c)  2001 Tassilo von Parseval. All rights reserved.  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL).
 


