#!perl

# AutoNap.pl - an automatizable Napster/OpenNap client
# Copyright (C) 2001  Tassilo v. Parseval
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Revision: 1.8 $, last change made $Date: 2001/05/15 06:07:39 $
# a routine for making some platform-dependant adoptions.

use strict;

mkdir "./target";
my $BINPATH = $ARGV[0] ||= '/usr/local/bin';
my $MANPATH = $ARGV[1] ||= '/usr/local/man';
my $LIBPATH = $ARGV[2] ||= '/usr/local/lib';
my $PERLPATH = $ENV{_};

open AUTONAP, "<AutoNap.PL" 			or die "Error: $!";
open AUTOCLIENT, "<AutoClient.PL"		or die "Error: $!";
open NAPEX, ">./target/AutoNap.pl"		or die "Error: $!";
open CLIENTEX, ">./target/AutoClient.pl"	or die "Error: $!";
open INSTALL, ">install.sh"			or die "Error: $!";

print "Examining your system...\n\n";
print "Where is Perl?\n";
print $PERLPATH."\n";
print "\n";
print "\n";
print "I will put my modules into:\n$LIBPATH\n";
print "\n";
print "After installation, AutoNap.pl will be in:\n";
print "$BINPATH\n";
print "\n";


#-----------------------------
# Generation of new executable
#-----------------------------
print "Generating executable of AutoNap.pl:\n";
while (<AUTONAP>) {
	my $l = $_;
	
	if ($l =~ /#!perl/) {
		print NAPEX "#! ".$ENV{_}." -w \n";
	}
	elsif ($l =~ /#use\slib/) {
		print NAPEX "use lib \"$LIBPATH/AutoNap\";\n";
	}
	else {
		print NAPEX $l;
	}
	print STDOUT ".";
}
print "\nGenerating executable of AutoClient.pl:\n";
while (<AUTOCLIENT>) {
	my $l = $_;

	if ($l =~ /#!perl/) {
		print CLIENTEX "#! ".$ENV{_}." -w \n";
	}
	else {
		print CLIENTEX $l;
	}
	print STDOUT ".";
}
				
close AUTONAP;
close AUTOCLIENT;
close AUTOEX;
close CLIENTEX;
print "\n";


#----------------------------
# Generation of installscript
#----------------------------
print "Generating install.sh:\n";
print INSTALL 
	"#!/bin/sh\n",
	"\n",
	"BINPATH='$BINPATH'\n",
	"MANPATH='$MANPATH'\n",
	"LIBPATH='$LIBPATH'\n",
	"CONFFILEPATH=\$HOME\n",
	"\n",
	"case \"\$1\" in\n",
	"      install)\n",
	" 	   		echo Making directory $LIBPATH/AutoNap...\n",
	"	     	mkdir $LIBPATH/AutoNap/\n",
	" 			echo Copying executables...\n",
	"			cp ./target/AutoNap.pl \$BINPATH\n",
	"	        cp ./target/AutoClient.pl \$BINPATH\n",
	"           echo Copying manpages and conffile...\n",
    "			cp AutoNap.pl.1 \$MANPATH/man1\n",
	"	        cp autonap.conf \$CONFFILEPATH\n",
	"			cp autonap.conf.5 \$MANPATH/man5\n",
	"           echo Copying AutoLib modules...\n",
	"			cp -r AutoLib/ \$LIBPATH/AutoNap\n",
	"           echo Making executables executable...\n",
	"			chmod 775 \$BINPATH/AutoNap.pl\n",
	"           chmod 775 \$BINPATH/AutoClient.pl\n",
	"      ;;\n",
	"      remove)\n",
	"			echo Removing executables \\($BINPATH/".
				"Auto\\[\\(Client\\|Nap\\)\\].pl\\)\n",
	"        	rm \$BINPATH/AutoNap.pl\n",
	"           rm \$BINPATH/AutoClient.pl\n",
	"           echo Removing manpages \\(in $MANPATH: man1/AutoNap.pl.1, ". 
			    "man5/autonap.conf.5\\)\n",
	"	        rm \$MANPATH/man1/AutoNap.pl.1\n",
	"	        rm \$MANPATH/man5/autonap.conf.5\n",
	"           echo Removing AutoNap modules \\($LIBPATH/AutoNap\\)\n",
	"	        rm -r \$LIBPATH/AutoNap\n",
	"           echo Finally removing conffile\n",
	"	        rm \$CONFFILEPATH/autonap.conf\n",
	"      ;;\n",
	"      *)\n",
	"		    echo \"usage: install.sh install | remove\"\n",
	"      ;;\n",
	"esac\n";
print "Generating of install.sh finished.\nYou can now install AutoNap".
	  " by invoking 'install.sh install'\n";

close INSTALL;

my $mode = 0755;
chmod $mode, 'install.sh';
